# remove when we don't support py38 anymore
from __future__ import annotations

from pytest import MonkeyPatch

from blag.quickstart import get_input, quickstart


def test_get_input_default_answer(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr('builtins.input', lambda x: '')
    answer = get_input("foo", "bar")
    assert answer == 'bar'


def test_get_input(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr('builtins.input', lambda x: 'baz')
    answer = get_input("foo", "bar")
    assert answer == 'baz'


def test_quickstart(cleandir: str, monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr('builtins.input', lambda x: 'foo')
    quickstart(None)
    with open('config.ini', 'r') as fh:
        data = fh.read()
    assert 'base_url = foo' in data
    assert 'title = foo' in data
    assert 'description = foo' in data
    assert 'author = foo' in data



# remove when we don't support py38 anymore
from __future__ import annotations
from datetime import datetime
from typing import Any

import pytest
import markdown

from blag.markdown import convert_markdown, markdown_factory


@pytest.mark.parametrize(
    "input_, expected",
    [
        # inline
        ('[test](test.md)', 'test.html'),
        ('[test](test.md "test")', 'test.html'),
        ('[test](a/test.md)', 'a/test.html'),
        ('[test](a/test.md "test")', 'a/test.html'),
        ('[test](/test.md)', '/test.html'),
        ('[test](/test.md "test")', '/test.html'),
        ('[test](/a/test.md)', '/a/test.html'),
        ('[test](/a/test.md "test")', '/a/test.html'),
        # reference
        ('[test][]\n[test]: test.md ' '', 'test.html'),
        ('[test][]\n[test]: test.md "test"', 'test.html'),
        ('[test][]\n[test]: a/test.md', 'a/test.html'),
        ('[test][]\n[test]: a/test.md "test"', 'a/test.html'),
        ('[test][]\n[test]: /test.md', '/test.html'),
        ('[test][]\n[test]: /test.md "test"', '/test.html'),
        ('[test][]\n[test]: /a/test.md', '/a/test.html'),
        ('[test][]\n[test]: /a/test.md "test"', '/a/test.html'),
    ],
)
def test_convert_markdown_links(input_: str, expected: str) -> None:
    md = markdown_factory()
    html, _ = convert_markdown(md, input_)
    assert expected in html


@pytest.mark.parametrize(
    "input_, expected",
    [
        # scheme
        ('[test](https://)', 'https://'),
        # netloc
        ('[test](//test.md)', '//test.md'),
        # no path
        ('[test]()', ''),
    ],
)
def test_dont_convert_normal_links(input_: str, expected: str) -> None:
    md = markdown_factory()
    html, _ = convert_markdown(md, input_)
    assert expected in html


@pytest.mark.parametrize(
    "input_, expected",
    [
        ('foo: bar', {'foo': 'bar'}),
        ('foo: those are several words', {'foo': 'those are several words'}),
        ('tags: this, is, a, test\n', {'tags': ['this', 'is', 'a', 'test']}),
        ('tags: this, IS, a, test', {'tags': ['this', 'is', 'a', 'test']}),
        (
            'date: 2020-01-01 12:10',
            {'date': datetime(2020, 1, 1, 12, 10).astimezone()},
        ),
    ],
)
def test_convert_metadata(input_: str, expected: dict[str, Any]) -> None:
    md = markdown_factory()
    _, meta = convert_markdown(md, input_)
    assert expected == meta


def test_markdown_factory() -> None:
    md = markdown_factory()
    assert isinstance(md, markdown.Markdown)


def test_smarty() -> None:
    md = markdown_factory()

    md1 = """

this --- is -- a test ...

    """
    html, meta = convert_markdown(md, md1)
    assert 'mdash' in html
    assert 'ndash' in html
    assert 'hellip' in html


def test_smarty_code() -> None:
    md = markdown_factory()

    md1 = """
```
this --- is -- a test ...
```
    """
    html, meta = convert_markdown(md, md1)
    assert 'mdash' not in html
    assert 'ndash' not in html
    assert 'hellip' not in html



# remove when we don't support py38 anymore
from __future__ import annotations
from argparse import Namespace
from typing import Iterator, Callable
from tempfile import TemporaryDirectory
import os

import pytest
from jinja2 import Environment, Template

from blag import blag


@pytest.fixture
def environment() -> Iterator[Environment]:
    site = {
        'base_url': 'site base_url',
        'title': 'site title',
        'description': 'site description',
        'author': 'site author',
    }
    env = blag.environment_factory(globals_=dict(site=site))
    yield env


@pytest.fixture
def page_template(environment: Environment) -> Iterator[Template]:
    yield environment.get_template('page.html')


@pytest.fixture
def article_template(environment: Environment) -> Iterator[Template]:
    yield environment.get_template('article.html')


@pytest.fixture
def archive_template(environment: Environment) -> Iterator[Template]:
    yield environment.get_template('archive.html')


@pytest.fixture
def tags_template(environment: Environment) -> Iterator[Template]:
    yield environment.get_template('tags.html')


@pytest.fixture
def tag_template(environment: Environment) -> Iterator[Template]:
    yield environment.get_template('tag.html')


@pytest.fixture
def cleandir() -> Iterator[str]:
    """Create a temporary workind directory and cwd."""
    config = """
[main]
base_url = https://example.com/
title = title
description = description
author = a. u. thor
    """

    with TemporaryDirectory() as dir:
        for d in 'content', 'build', 'static', 'templates':
            os.mkdir(f'{dir}/{d}')
        with open(f'{dir}/config.ini', 'w') as fh:
            fh.write(config)
        # change directory
        old_cwd = os.getcwd()
        os.chdir(dir)
        yield dir
        # and change back afterwards
        os.chdir(old_cwd)


@pytest.fixture
def args(cleandir: Callable[[], Iterator[str]]) -> Iterator[Namespace]:

    args = Namespace(
        input_dir='content',
        output_dir='build',
        static_dir='static',
        template_dir='templates',
    )
    yield args



#!/usr/bin/env python3

"""blag's core methods.

"""

# remove when we don't support py38 anymore
from __future__ import annotations
from typing import Any
import argparse
import os
import shutil
import logging
import configparser
import sys

from jinja2 import (
    Environment,
    ChoiceLoader,
    FileSystemLoader,
    PackageLoader,
    Template,
)
import feedgenerator

from blag.markdown import markdown_factory, convert_markdown
from blag.devserver import serve
from blag.version import __VERSION__
from blag.quickstart import quickstart

logger = logging.getLogger(__name__)
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(levelname)s %(name)s %(message)s',
)


def main(arguments: list[str] | None = None) -> None:
    """Main entrypoint for the CLI.

    This method parses the CLI arguments and executes the respective
    commands.

    Parameters
    ----------
    arguments
        optional parameters, used for testing

    """
    args = parse_args(arguments)
    # set loglevel
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    logger.debug(f"This is blag {__VERSION__}.")
    args.func(args)


def parse_args(args: list[str] | None = None) -> argparse.Namespace:
    """Parse command line arguments.

    Parameters
    ----------
    args
        optional parameters, used for testing

    Returns
    -------
    arparse.Namespace

    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--version',
        action='version',
        version='%(prog)s ' + __VERSION__,
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        help='Verbose output.',
    )

    commands = parser.add_subparsers(dest='command')
    commands.required = True

    build_parser = commands.add_parser(
        'build',
        help='Build website.',
    )
    build_parser.set_defaults(func=build)
    build_parser.add_argument(
        '-i',
        '--input-dir',
        default='content',
        help='Input directory (default: content)',
    )
    build_parser.add_argument(
        '-o',
        '--output-dir',
        default='build',
        help='Ouptut directory (default: build)',
    )
    build_parser.add_argument(
        '-t',
        '--template-dir',
        default='templates',
        help='Template directory (default: templates)',
    )
    build_parser.add_argument(
        '-s',
        '--static-dir',
        default='static',
        help='Static directory (default: static)',
    )

    quickstart_parser = commands.add_parser(
        'quickstart',
        help="Quickstart blag, creating necessary configuration.",
    )
    quickstart_parser.set_defaults(func=quickstart)

    serve_parser = commands.add_parser(
        'serve',
        help="Start development server.",
    )
    serve_parser.set_defaults(func=serve)
    serve_parser.add_argument(
        '-i',
        '--input-dir',
        default='content',
        help='Input directory (default: content)',
    )
    serve_parser.add_argument(
        '-o',
        '--output-dir',
        default='build',
        help='Ouptut directory (default: build)',
    )
    serve_parser.add_argument(
        '-t',
        '--template-dir',
        default='templates',
        help='Template directory (default: templates)',
    )
    serve_parser.add_argument(
        '-s',
        '--static-dir',
        default='static',
        help='Static directory (default: static)',
    )

    return parser.parse_args(args)


def get_config(configfile: str) -> configparser.SectionProxy:
    """Load site configuration from configfile.

    Parameters
    ----------
    configfile
        path to configuration file


    Returns
    -------
    configparser.SectionProxy

    """
    config = configparser.ConfigParser()
    config.read(configfile)
    # check for the mandatory options
    for value in 'base_url', 'title', 'description', 'author':
        try:
            config['main'][value]
        except Exception:
            print(f'{value} is missing in {configfile}!')
            sys.exit(1)

    if not config['main']['base_url'].endswith('/'):
        logger.warning('base_url does not end with a slash, adding it.')
        config['main']['base_url'] += '/'

    return config['main']


def environment_factory(
    template_dir: str | None = None,
    globals_: dict[str, object] | None = None,
) -> Environment:
    """Environment factory.

    Creates a Jinja2 Environment with the default templates and
    additional templates from `template_dir` loaded. If `globals` are
    provided, they are attached to the environment and thus available to
    all contexts.

    Parameters
    ----------
    template_dir
        directory containing the templates
    globals_

    Returns
    -------
    jinja2.Environment

    """
    # first we try the custom templates, and fall back the ones provided
    # by blag
    loaders: list[FileSystemLoader | PackageLoader] = []
    if template_dir:
        loaders.append(FileSystemLoader([template_dir]))
    loaders.append(PackageLoader('blag', 'templates'))
    env = Environment(loader=ChoiceLoader(loaders))
    if globals_:
        env.globals = globals_
    return env


def build(args: argparse.Namespace) -> None:
    """Build the site.

    This is blag's main method that builds the site, generates the feed
    etc.

    Parameters
    ----------
    args

    """
    os.makedirs(f'{args.output_dir}', exist_ok=True)
    convertibles = []
    for root, dirnames, filenames in os.walk(args.input_dir):
        for filename in filenames:
            rel_src = os.path.relpath(
                f'{root}/{filename}', start=args.input_dir
            )
            # all non-markdown files are just copied over, the markdown
            # files are converted to html
            if rel_src.endswith('.md'):
                rel_dst = rel_src
                rel_dst = rel_dst[:-3] + '.html'
                convertibles.append((rel_src, rel_dst))
            else:
                shutil.copy(
                    f'{args.input_dir}/{rel_src}',
                    f'{args.output_dir}/{rel_src}',
                )
        for dirname in dirnames:
            # all directories are copied into the output directory
            path = os.path.relpath(f'{root}/{dirname}', start=args.input_dir)
            os.makedirs(f'{args.output_dir}/{path}', exist_ok=True)

    # copy static files over
    logger.info('Copying static files.')
    if os.path.exists(args.static_dir):
        shutil.copytree(args.static_dir, args.output_dir, dirs_exist_ok=True)

    config = get_config('config.ini')

    env = environment_factory(args.template_dir, dict(site=config))

    page_template = env.get_template('page.html')
    article_template = env.get_template('article.html')
    archive_template = env.get_template('archive.html')
    tags_template = env.get_template('tags.html')
    tag_template = env.get_template('tag.html')

    articles, pages = process_markdown(
        convertibles,
        args.input_dir,
        args.output_dir,
        page_template,
        article_template,
    )

    generate_feed(
        articles,
        args.output_dir,
        base_url=config['base_url'],
        blog_title=config['title'],
        blog_description=config['description'],
        blog_author=config['author'],
    )
    generate_archive(articles, archive_template, args.output_dir)
    generate_tags(articles, tags_template, tag_template, args.output_dir)


def process_markdown(
    convertibles: list[tuple[str, str]],
    input_dir: str,
    output_dir: str,
    page_template: Template,
    article_template: Template,
) -> tuple[list[tuple[str, dict[str, Any]]], list[tuple[str, dict[str, Any]]]]:
    """Process markdown files.

    This method processes the convertibles, converts them to html and
    saves them to the respective destination paths.

    If a markdown file has a `date` metadata field it will be recognized
    as article otherwise as page.

    Parameters
    ----------
    convertibles
        relative paths to markdown- (src) html- (dest) files
    input_dir
    output_dir
    page_template, archive_template
        templats for pages and articles

    Returns
    -------
    list[tuple[str, dict[str, Any]]], list[tuple[str, dict[str, Any]]]
        articles and pages

    """
    logger.info("Converting Markdown files...")
    md = markdown_factory()

    articles = []
    pages = []
    for src, dst in convertibles:
        logger.debug(f'Processing {src}')

        with open(f'{input_dir}/{src}', 'r') as fh:
            body = fh.read()

        content, meta = convert_markdown(md, body)

        context = dict(content=content)
        context.update(meta)

        # if markdown has date in meta, we treat it as a blog article,
        # everything else are just pages
        if meta and 'date' in meta:
            articles.append((dst, context))
            result = article_template.render(context)
        else:
            pages.append((dst, context))
            result = page_template.render(context)
        with open(f'{output_dir}/{dst}', 'w') as fh_dest:
            fh_dest.write(result)

    # sort articles by date, descending
    articles = sorted(articles, key=lambda x: x[1]['date'], reverse=True)
    return articles, pages


def generate_feed(
    articles: list[tuple[str, dict[str, Any]]],
    output_dir: str,
    base_url: str,
    blog_title: str,
    blog_description: str,
    blog_author: str,
) -> None:
    """Generate Atom feed.

    Parameters
    ----------
    articles
        list of relative output path and article dictionary
    output_dir
        where the feed is stored
    base_url
        base url
    blog_title
        blog title
    blog_description
        blog description
    blog_author
        blog author

    """
    logger.info('Generating Atom feed.')
    feed = feedgenerator.Atom1Feed(
        link=base_url,
        title=blog_title,
        description=blog_description,
        feed_url=base_url + 'atom.xml',
    )

    for dst, context in articles:
        # if article has a description, use that. otherwise fall back to
        # the title
        description = context.get('description', context['title'])

        feed.add_item(
            title=context['title'],
            author_name=blog_author,
            link=base_url + dst,
            description=description,
            content=context['content'],
            pubdate=context['date'],
        )

    with open(f'{output_dir}/atom.xml', 'w') as fh:
        feed.write(fh, encoding='utf8')


def generate_archive(
    articles: list[tuple[str, dict[str, Any]]],
    template: Template,
    output_dir: str,
) -> None:
    """Generate the archive page.

    Parameters
    ----------
    articles
        List of articles. Each article has the destination path and a
        dictionary with the content.
    template
    output_dir

    """
    archive = []
    for dst, context in articles:
        entry = context.copy()
        entry['dst'] = dst
        archive.append(entry)

    result = template.render(dict(archive=archive))
    with open(f'{output_dir}/index.html', 'w') as fh:
        fh.write(result)


def generate_tags(
    articles: list[tuple[str, dict[str, Any]]],
    tags_template: Template,
    tag_template: Template,
    output_dir: str,
) -> None:
    """Generate the tags page.

    Parameters
    ----------
    articles
        List of articles. Each article has the destination path and a
        dictionary with the content.
    tags_template, tag_template
    output_dir

    """
    logger.info("Generating Tag-pages.")
    os.makedirs(f'{output_dir}/tags', exist_ok=True)
    # get tags number of occurrences
    all_tags: dict[str, int] = {}
    for _, context in articles:
        tags: list[str] = context.get('tags', [])
        for tag in tags:
            all_tags[tag] = all_tags.get(tag, 0) + 1
    # sort by occurrence
    taglist: list[tuple[str, int]] = sorted(
        all_tags.items(), key=lambda x: x[1], reverse=True
    )

    result = tags_template.render(dict(tags=taglist))
    with open(f'{output_dir}/tags/index.html', 'w') as fh:
        fh.write(result)

    # get tags and archive per tag
    all_tags2: dict[str, list[dict[str, Any]]] = {}
    for dst, context in articles:
        tags = context.get('tags', [])
        for tag in tags:
            archive: list[dict[str, Any]] = all_tags2.get(tag, [])
            entry = context.copy()
            entry['dst'] = dst
            archive.append(entry)
            all_tags2[tag] = archive

    for tag, archive in all_tags2.items():
        result = tag_template.render(dict(archive=archive, tag=tag))
        with open(f'{output_dir}/tags/{tag}.html', 'w') as fh:
            fh.write(result)


if __name__ == '__main__':
    main()



# remove when we don't support py38 anymore
from __future__ import annotations
import datetime

from jinja2 import Template


def test_page(page_template: Template) -> None:
    ctx = {
        'content': 'this is the content',
        'title': 'this is the title',
    }
    result = page_template.render(ctx)
    assert 'this is the content' in result
    assert 'this is the title' in result


def test_article(article_template: Template) -> None:
    ctx = {
        'content': 'this is the content',
        'title': 'this is the title',
        'date': datetime.datetime(1980, 5, 9),
    }
    result = article_template.render(ctx)
    assert 'this is the content' in result
    assert 'this is the title' in result
    assert '1980-05-09' in result


def test_archive(archive_template: Template) -> None:
    entry = {
        'title': 'this is a title',
        'dst': 'https://example.com/link',
        'date': datetime.datetime(1980, 5, 9),
    }
    archive = [entry]
    ctx = {
        'archive': archive,
    }
    result = archive_template.render(ctx)
    assert 'site title' in result

    assert 'this is a title' in result
    assert '1980-05-09' in result
    assert 'https://example.com/link' in result


def test_tags(tags_template: Template) -> None:
    tags = [('foo', 42)]
    ctx = {
        'tags': tags,
    }
    result = tags_template.render(ctx)
    assert 'Tags' in result

    assert 'foo.html' in result
    assert 'foo' in result
    assert '42' in result


def test_tag(tag_template: Template) -> None:
    entry = {
        'title': 'this is a title',
        'dst': 'https://example.com/link',
        'date': datetime.datetime(1980, 5, 9),
    }
    archive = [entry]
    ctx = {
        'tag': 'foo',
        'archive': archive,
    }
    result = tag_template.render(ctx)
    assert 'Tag foo' in result

    assert 'this is a title' in result
    assert '1980-05-09' in result
    assert 'https://example.com/link' in result



"""Markdown Processing.

This module contains the methods responsible for blag's markdown
processing.

"""

# remove when we don't support py38 anymore
from __future__ import annotations
from datetime import datetime
import logging
from urllib.parse import urlsplit, urlunsplit
from xml.etree.ElementTree import Element

from markdown import Markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor


logger = logging.getLogger(__name__)


def markdown_factory() -> Markdown:
    """Create a Markdown instance.

    This method exists only to ensure we use the same Markdown instance
    for tests as for the actual thing.

    Returns
    -------
    markdown.Markdown

    """
    md = Markdown(
        extensions=[
            'meta',
            'fenced_code',
            'codehilite',
            'smarty',
            MarkdownLinkExtension(),
        ],
        output_format='html',
    )
    return md


def convert_markdown(
    md: Markdown,
    markdown: str,
) -> tuple[str, dict[str, str]]:
    """Convert markdown into html and extract meta data.

    Some meta data is treated special:
        * `date` is converted into datetime with local timezone
        * `tags` is interpreted as a comma-separeted list of strings.
          All strings are stripped and converted to lower case.

    Parameters
    ----------
    md
        the Markdown instance
    markdown
        the markdown text that should be converted

    Returns
    -------
    str, dict[str, str]
        html and metadata

    """
    md.reset()
    content = md.convert(markdown)
    meta = md.Meta  # type: ignore

    # markdowns metadata consists as list of strings -- one item per
    # line. let's convert into single strings.
    for key, value in meta.items():
        value = '\n'.join(value)
        meta[key] = value

    # convert known metadata
    # date: datetime
    if 'date' in meta:
        meta['date'] = datetime.fromisoformat(meta['date'])
        meta['date'] = meta['date'].astimezone()
    # tags: list[str] and lower case
    if 'tags' in meta:
        tags = meta['tags'].split(',')
        tags = [t.lower() for t in tags]
        tags = [t.strip() for t in tags]
        meta['tags'] = tags

    return content, meta


class MarkdownLinkTreeprocessor(Treeprocessor):
    """Converts relative links to .md files to .html"""

    def run(self, root: Element) -> Element:
        for element in root.iter():
            if element.tag == 'a':
                url = element.get('href')
                # element.get could also return None, we haven't seen this so
                # far, so lets wait if we raise this
                assert url is not None
                url = str(url)
                converted = self.convert(url)
                element.set('href', converted)
        return root

    def convert(self, url: str) -> str:
        scheme, netloc, path, query, fragment = urlsplit(url)
        logger.debug(
            f'{url}: {scheme=} {netloc=} {path=} {query=} {fragment=}'
        )
        if scheme or netloc or not path:
            return url
        if path.endswith('.md'):
            path = path[:-3] + '.html'

        url = urlunsplit((scheme, netloc, path, query, fragment))
        return url


class MarkdownLinkExtension(Extension):
    """markdown.extension that converts relative .md- to .html-links."""

    def extendMarkdown(self, md: Markdown) -> None:
        md.treeprocessors.register(
            MarkdownLinkTreeprocessor(md),
            'mdlink',
            0,
        )



# remove when we don't support py38 anymore
from __future__ import annotations
import time
import threading
from argparse import Namespace

import pytest

from blag import devserver


def test_get_last_modified(cleandir: str) -> None:
    # take initial time
    t1 = devserver.get_last_modified(['content'])

    # wait a bit, create a file and measure again
    time.sleep(0.1)
    with open('content/test', 'w') as fh:
        fh.write('boo')
    t2 = devserver.get_last_modified(['content'])

    # wait a bit and take time again
    time.sleep(0.1)
    t3 = devserver.get_last_modified(['content'])

    assert t2 > t1
    assert t2 == t3


def test_autoreload_builds_immediately(args: Namespace) -> None:
    # create a dummy file that can be build
    with open('content/test.md', 'w') as fh:
        fh.write('boo')

    t = threading.Thread(
        target=devserver.autoreload,
        args=(args,),
        daemon=True,
    )
    t0 = devserver.get_last_modified(['build'])
    t.start()
    # try for 5 seconds...
    for i in range(5):
        time.sleep(1)
        t1 = devserver.get_last_modified(['build'])
        print(t1)
        if t1 > t0:
            break
    assert t1 > t0


@pytest.mark.filterwarnings(
    "ignore::pytest.PytestUnhandledThreadExceptionWarning"
)
def test_autoreload(args: Namespace) -> None:
    t = threading.Thread(
        target=devserver.autoreload,
        args=(args,),
        daemon=True,
    )
    t.start()

    t0 = devserver.get_last_modified(['build'])

    # create a dummy file that can be build
    with open('content/test.md', 'w') as fh:
        fh.write('boo')

    # try for 5 seconds to see if we rebuild once...
    for i in range(5):
        time.sleep(1)
        t1 = devserver.get_last_modified(['build'])
        if t1 > t0:
            break
    assert t1 > t0



"""Helper methods for blag's quickstart command.

"""

# remove when we don't support py38 anymore
from __future__ import annotations
import configparser
import argparse


def get_input(question: str, default: str) -> str:
    """Prompt for user input.

    This is a wrapper around the input-builtin. It will show the default answer
    in the prompt and -- if no answer was given -- use the default.

    Parameters
    ----------
    question
        the question the user is presented
    default
        the default value that will be used if no answer was given

    Returns
    -------
    str
        the answer

    """
    reply = input(f"{question} [{default}]: ")
    if not reply:
        reply = default
    return reply


def quickstart(args: argparse.Namespace | None) -> None:
    """Quickstart.

    This method asks the user some questions and generates a
    configuration file that is needed in order to run blag.

    Parameters
    ----------
    args
        not used

    """
    base_url = get_input(
        "Hostname (and path) to the root?",
        "https://example.com/",
    )
    title = get_input(
        "Title of your website?",
        "My little blog",
    )
    description = get_input(
        "Description of your website?",
        "John Doe's Blog",
    )
    author = get_input(
        "Author of your website",
        "John Doe",
    )

    config = configparser.ConfigParser()
    config['main'] = {
        'base_url': base_url,
        'title': title,
        'description': description,
        'author': author,
    }
    with open('config.ini', 'w') as fh:
        config.write(fh)



# remove when we don't support py38 anymore
from __future__ import annotations
from tempfile import TemporaryDirectory
import os
from datetime import datetime
from typing import Any
from argparse import Namespace

import pytest
from pytest import CaptureFixture, LogCaptureFixture
from jinja2 import Template

from blag import __VERSION__
from blag import blag


def test_generate_feed(cleandir: str) -> None:
    articles: list[tuple[str, dict[str, Any]]] = []
    blag.generate_feed(articles, 'build', ' ', ' ', ' ', ' ')
    assert os.path.exists('build/atom.xml')


def test_feed(cleandir: str) -> None:
    articles: list[tuple[str, dict[str, Any]]] = [
        (
            'dest1.html',
            {
                'title': 'title1',
                'date': datetime(2019, 6, 6),
                'content': 'content1',
            },
        ),
        (
            'dest2.html',
            {
                'title': 'title2',
                'date': datetime(1980, 5, 9),
                'content': 'content2',
            },
        ),
    ]

    blag.generate_feed(
        articles,
        'build',
        'https://example.com/',
        'blog title',
        'blog description',
        'blog author',
    )
    with open('build/atom.xml') as fh:
        feed = fh.read()

    assert '<title>blog title</title>' in feed
    # enable when https://github.com/getpelican/feedgenerator/issues/22
    # is fixed
    # assert '<subtitle>blog description</subtitle>' in feed
    assert '<author><name>blog author</name></author>' in feed

    # article 1
    assert '<title>title1</title>' in feed
    assert '<summary type="html">title1' in feed
    assert '<published>2019-06-06' in feed
    assert '<content type="html">content1' in feed
    assert '<link href="https://example.com/dest1.html"' in feed

    # article 2
    assert '<title>title2</title>' in feed
    assert '<summary type="html">title2' in feed
    assert '<published>1980-05-09' in feed
    assert '<content type="html">content2' in feed
    assert '<link href="https://example.com/dest2.html"' in feed


def test_generate_feed_with_description(cleandir: str) -> None:
    # if a description is provided, it will be used as the summary in
    # the feed, otherwise we simply use the title of the article
    articles: list[tuple[str, dict[str, Any]]] = [
        (
            'dest.html',
            {
                'title': 'title',
                'description': 'description',
                'date': datetime(2019, 6, 6),
                'content': 'content',
            },
        )
    ]
    blag.generate_feed(articles, 'build', ' ', ' ', ' ', ' ')

    with open('build/atom.xml') as fh:
        feed = fh.read()

    assert '<title>title</title>' in feed
    assert '<summary type="html">description' in feed
    assert '<published>2019-06-06' in feed
    assert '<content type="html">content' in feed


def test_parse_args_build() -> None:
    # test default args
    args = blag.parse_args(['build'])
    assert args.input_dir == 'content'
    assert args.output_dir == 'build'
    assert args.template_dir == 'templates'
    assert args.static_dir == 'static'

    # input dir
    args = blag.parse_args(['build', '-i', 'foo'])
    assert args.input_dir == 'foo'
    args = blag.parse_args(['build', '--input-dir', 'foo'])
    assert args.input_dir == 'foo'

    # output dir
    args = blag.parse_args(['build', '-o', 'foo'])
    assert args.output_dir == 'foo'
    args = blag.parse_args(['build', '--output-dir', 'foo'])
    assert args.output_dir == 'foo'

    # template dir
    args = blag.parse_args(['build', '-t', 'foo'])
    assert args.template_dir == 'foo'
    args = blag.parse_args(['build', '--template-dir', 'foo'])
    assert args.template_dir == 'foo'

    # static dir
    args = blag.parse_args(['build', '-s', 'foo'])
    assert args.static_dir == 'foo'
    args = blag.parse_args(['build', '--static-dir', 'foo'])
    assert args.static_dir == 'foo'


def test_get_config() -> None:
    config = """
[main]
base_url = https://example.com/
title = title
description = description
author = a. u. thor
    """
    # happy path
    with TemporaryDirectory() as dir:
        configfile = f'{dir}/config.ini'
        with open(configfile, 'w') as fh:
            fh.write(config)

        config_parsed = blag.get_config(configfile)
        assert config_parsed['base_url'] == 'https://example.com/'
        assert config_parsed['title'] == 'title'
        assert config_parsed['description'] == 'description'
        assert config_parsed['author'] == 'a. u. thor'

    # a missing required config causes a sys.exit
    for x in 'base_url', 'title', 'description', 'author':
        config2 = '\n'.join(
            [line for line in config.splitlines() if not line.startswith(x)]
        )
        with TemporaryDirectory() as dir:
            configfile = f'{dir}/config.ini'
            with open(configfile, 'w') as fh:
                fh.write(config2)
            with pytest.raises(SystemExit):
                config_parsed = blag.get_config(configfile)

    # base_url gets / appended if it is missing
    config = """
[main]
base_url = https://example.com
title = title
description = description
author = a. u. thor
    """
    with TemporaryDirectory() as dir:
        configfile = f'{dir}/config.ini'
        with open(configfile, 'w') as fh:
            fh.write(config)

        config_parsed = blag.get_config(configfile)
        assert config_parsed['base_url'] == 'https://example.com/'


def test_environment_factory() -> None:
    globals_: dict[str, object] = {'foo': 'bar', 'test': 'me'}
    env = blag.environment_factory(globals_=globals_)
    assert env.globals['foo'] == 'bar'
    assert env.globals['test'] == 'me'


def test_process_markdown(
    cleandir: str,
    page_template: Template,
    article_template: Template,
) -> None:
    page1 = """\
title: some page

some text
foo bar
    """

    article1 = """\
title: some article1
date: 2020-01-01

some text
foo bar
    """

    article2 = """\
title: some article2
date: 2021-01-01

some text
foo bar
    """

    convertibles = []
    for i, txt in enumerate((page1, article1, article2)):
        with open(f'content/{str(i)}', 'w') as fh:
            fh.write(txt)
        convertibles.append((str(i), str(i)))

    articles, pages = blag.process_markdown(
        convertibles, 'content', 'build', page_template, article_template
    )

    assert isinstance(articles, list)
    assert len(articles) == 2
    for dst, context in articles:
        assert isinstance(dst, str)
        assert isinstance(context, dict)
        assert 'content' in context

    assert isinstance(pages, list)
    assert len(pages) == 1
    for dst, context in pages:
        assert isinstance(dst, str)
        assert isinstance(context, dict)
        assert 'content' in context


def test_build(args: Namespace) -> None:
    page1 = """\
title: some page

some text
foo bar
    """

    article1 = """\
title: some article1
date: 2020-01-01
tags: foo, bar

some text
foo bar
    """

    article2 = """\
title: some article2
date: 2021-01-01
tags: baz

some text
foo bar
    """

    # write some convertibles
    convertibles = []
    for i, txt in enumerate((page1, article1, article2)):
        with open(f'{args.input_dir}/{str(i)}.md', 'w') as fh:
            fh.write(txt)
        convertibles.append((str(i), str(i)))

    # some static files
    with open(f'{args.static_dir}/test', 'w') as fh:
        fh.write('hello')

    os.mkdir(f'{args.input_dir}/testdir')
    with open(f'{args.input_dir}/testdir/test', 'w') as fh:
        fh.write('hello')

    blag.build(args)

    # test existence of the three converted files
    for i in range(3):
        assert os.path.exists(f'{args.output_dir}/{i}.html')
    # ... static file
    assert os.path.exists(f'{args.output_dir}/test')
    # ... directory
    assert os.path.exists(f'{args.output_dir}/testdir/test')
    # ... feed
    assert os.path.exists(f'{args.output_dir}/atom.xml')
    # ... archive
    assert os.path.exists(f'{args.output_dir}/index.html')
    # ... tags
    assert os.path.exists(f'{args.output_dir}/tags/index.html')
    assert os.path.exists(f'{args.output_dir}/tags/foo.html')
    assert os.path.exists(f'{args.output_dir}/tags/bar.html')


def test_main(cleandir: str) -> None:
    blag.main(['build'])


def test_cli_version(capsys: CaptureFixture[str]) -> None:
    with pytest.raises(SystemExit) as ex:
        blag.main(['--version'])
    # normal system exit
    assert ex.value.code == 0
    # proper version reported
    out, _ = capsys.readouterr()
    assert __VERSION__ in out


def test_cli_verbose(cleandir: str, caplog: LogCaptureFixture) -> None:
    blag.main(['build'])
    assert 'DEBUG' not in caplog.text

    blag.main(['--verbose', 'build'])
    assert 'DEBUG' in caplog.text
